use models::data::Data;
use models::extra::Extra;
use regex::Regex;
use serde::{Deserialize, Deserializer};
use std::collections::BTreeMap;
use std::fmt;

#[derive(Hash)]
#[derive(Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Content {
    #[serde(skip_deserializing)]
    pub id: u64,
    #[serde(deserialize_with = "normalize_title")]
    pub title: String,
    #[serde(skip_deserializing)]
    #[serde(rename(deserialize = "cdKey"))]
    pub cd_keys: BTreeMap<String, String>,
    #[serde(skip_deserializing)]
    pub is_movie: bool,
    #[serde(skip_deserializing)]
    pub data: Vec<Data>,
    pub extras: Vec<Extra>,
}

impl fmt::Display for Content {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "({})", self.title)
    }
}

fn normalize_title<D>(deserializer: D) -> Result<String, D::Error>
    where D: Deserializer
{
    let raw_title = String::deserialize(deserializer)?.replace(":", " - ");

    let regex_normalize = Regex::new(r"[^'^\w^\s^-]+").unwrap();
    let regex_whitespace = Regex::new(r"\s\s+").unwrap();

    let title_normalized = regex_normalize
        .replace_all(raw_title.as_str(), "")
        .to_string();

    let title_whitespace = regex_whitespace
        .replace_all(title_normalized.as_str(), " ")
        .to_string();

    Ok(title_whitespace)
}
